@@ -645,7 +645,7 @@ struct mnt_entry {
 };
 
 /*
- * Find first occurence of up an option string (as "option=") in @options,
+ * Find first occurrence of up an option string (as "option=") in @options,
  * separated by comma. Return allocated string as "option=value"
  */
 static char *find_option(const char *options, const char *option)
@@ -818,7 +818,7 @@ static void parse_mntinfo_line(char *line, struct mnt_entry *ent)
  *   37 29 0:32 /vol/dir2 /othermnt ro,relatime - btrfs /dev/sda2 ro,ssd,space_cache,subvolid=256,subvol=/vol
  *
  * If we try to find a mount point only using subvol and subvolid from mount
- * options we would get mislead to belive that /othermnt has the same content
+ * options we would get mislead to believe that /othermnt has the same content
  * as /mnt.
  *
  * But, using mountinfo, we have the pathaname _inside_ the filesystem, so we
@@ -1477,6 +1477,7 @@ int sysfs_read_file(int fd, char *buf, size_t size)
 static const char exclop_def[][16] = {
 	[BTRFS_EXCLOP_NONE]		= "none",
 	[BTRFS_EXCLOP_BALANCE]		= "balance",
+	[BTRFS_EXCLOP_BALANCE_PAUSED]	= "balance paused",
 	[BTRFS_EXCLOP_DEV_ADD]		= "device add",
 	[BTRFS_EXCLOP_DEV_REMOVE]	= "device remove",
 	[BTRFS_EXCLOP_DEV_REPLACE]	= "device replace",
@@ -1554,6 +1555,15 @@ int check_running_fs_exclop(int fd, enum exclusive_operation start, bool enqueue
 		goto out;
 	}
 
+	/*
+	 * Some combinations are compatible:
+	 * - start device add when balance is paused (kernel 5.17)
+	 */
+	if (start == BTRFS_EXCLOP_DEV_ADD && exclop == BTRFS_EXCLOP_BALANCE_PAUSED) {
+		ret = 0;
+		goto out;
+	}
+
 	if (!enqueue) {
 		error(
 	"unable to start %s, another exclusive operation '%s' in progress",
